@@ -212,6 +212,7 @@ SQLRETURN SQLColAttributeW ( SQLHSTMT statement_handle,
 
         return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
     }
+    /*
     else if ( statement -> state == STATE_S2 &&
             field_identifier != SQL_DESC_COUNT )
     {
@@ -227,6 +228,7 @@ SQLRETURN SQLColAttributeW ( SQLHSTMT statement_handle,
 
         return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
     }
+    */
     else if ( statement -> state == STATE_S4 )
     {
         dm_log_write( __FILE__, 
@@ -243,7 +245,10 @@ SQLRETURN SQLColAttributeW ( SQLHSTMT statement_handle,
     }
     else if ( statement -> state == STATE_S8 ||
             statement -> state == STATE_S9 ||
-            statement -> state == STATE_S10 )
+            statement -> state == STATE_S10 ||
+            statement -> state == STATE_S13 ||
+            statement -> state == STATE_S14 ||
+            statement -> state == STATE_S15 )
     {
         dm_log_write( __FILE__, 
                 __LINE__, 
@@ -388,7 +393,7 @@ SQLRETURN SQLColAttributeW ( SQLHSTMT statement_handle,
                   case SQL_DESC_NAME:
                     if ( SQL_SUCCEEDED( ret ) && character_attribute && as1 )
                     {
-                        ansi_to_unicode_copy( character_attribute, (char*) as1, SQL_NTS, statement -> connection );
+                        ansi_to_unicode_copy( character_attribute, (char*) as1, SQL_NTS, statement -> connection, NULL );
 					}
 					if ( SQL_SUCCEEDED( ret ) && string_length )
 					{
@@ -467,7 +472,7 @@ SQLRETURN SQLColAttributeW ( SQLHSTMT statement_handle,
               case SQL_COLUMN_NAME:
                 if ( SQL_SUCCEEDED( ret ) && character_attribute && as1 )
                 {
-                    ansi_to_unicode_copy( character_attribute, (char*) as1, SQL_NTS, statement -> connection );
+                    ansi_to_unicode_copy( character_attribute, (char*) as1, SQL_NTS, statement -> connection, NULL );
 				}
 				if ( SQL_SUCCEEDED( ret ) && string_length )
 				{
