@@ -4,7 +4,7 @@
  * (pharvey@codebydesign.com).
  *
  * Modified and extended by Nick Gorham
- * (nick@easysoft.com).
+ * (nick@lurcher.org).
  *
  * Any bugs or problems should be considered the fault of Nick and not
  * Peter.
@@ -27,9 +27,12 @@
  *
  **********************************************************************
  *
- * $Id: SQLDataSourcesW.c,v 1.5 2003/10/30 18:20:45 lurcher Exp $
+ * $Id: SQLDataSourcesW.c,v 1.6 2009/02/18 17:59:08 lurcher Exp $
  *
  * $Log: SQLDataSourcesW.c,v $
+ * Revision 1.6  2009/02/18 17:59:08  lurcher
+ * Shift to using config.h, the compile lines were making it hard to spot warnings
+ *
  * Revision 1.5  2003/10/30 18:20:45  lurcher
  *
  * Fix broken thread protection
@@ -72,11 +75,12 @@
  *
  **********************************************************************/
 
+#include <config.h>
 #include "drivermanager.h"
 
 static char const rcsid[]= "$RCSfile: SQLDataSourcesW.c,v $";
 
-#define BUFFERSIZE      1024
+#define BUFFERSIZE      1024 * 4
 
 SQLRETURN SQLDataSourcesW( SQLHENV environment_handle,
            SQLUSMALLINT direction,
@@ -95,13 +99,6 @@ SQLRETURN SQLDataSourcesW( SQLHENV environment_handle,
     char driver[ INI_MAX_PROPERTY_VALUE + 1 ];
     SQLCHAR s1[ 100 + LOG_MESSAGE_LEN ];
 
-    /*
-     * restrict for unicode
-     */
-
-    buffer_length1 = buffer_length1 / 2;
-    buffer_length2 = buffer_length2 / 2;
-
     if ( !__validate_env( environment ))
     {
         dm_log_write( __FILE__, 
@@ -118,7 +115,7 @@ SQLRETURN SQLDataSourcesW( SQLHENV environment_handle,
     if ( log_info.log_flag )
     {
         sprintf( environment -> msg, "\n\t\tEntry:\
-            \n\t\t\tEnvironment = %p",
+\n\t\t\tEnvironment = %p",
                 environment );
 
         dm_log_write( __FILE__, 
@@ -262,7 +259,7 @@ SQLRETURN SQLDataSourcesW( SQLHENV environment_handle,
         {
             SQLWCHAR *s1;
 
-            s1 = ansi_to_unicode_alloc((SQLCHAR*) object, SQL_NTS, NULL );
+            s1 = ansi_to_unicode_alloc((SQLCHAR*) object, SQL_NTS, NULL, NULL );
 
             if ( s1 )
             {
@@ -284,7 +281,7 @@ SQLRETURN SQLDataSourcesW( SQLHENV environment_handle,
         {
             SQLWCHAR *s1;
 
-            s1 = ansi_to_unicode_alloc((SQLCHAR*) property, SQL_NTS, NULL );
+            s1 = ansi_to_unicode_alloc((SQLCHAR*) property, SQL_NTS, NULL, NULL );
 
             if ( s1 )
             {
