@@ -3,7 +3,7 @@
  * (pharvey@codebydesign.com).
  *
  * Modified and extended by Nick Gorham
- * (nick@easysoft.com).
+ * (nick@lurcher.org).
  *
  * copyright (c) 1999 Nick Gorham
  *
@@ -26,9 +26,12 @@
  *
  **********************************************************************
  *
- * $Id: SQLExecDirect.c,v 1.10 2006/04/11 10:22:56 lurcher Exp $
+ * $Id: SQLExecDirect.c,v 1.11 2009/02/18 17:59:08 lurcher Exp $
  *
  * $Log: SQLExecDirect.c,v $
+ * Revision 1.11  2009/02/18 17:59:08  lurcher
+ * Shift to using config.h, the compile lines were making it hard to spot warnings
+ *
  * Revision 1.10  2006/04/11 10:22:56  lurcher
  * Fix a data type check
  *
@@ -166,9 +169,10 @@
  *
  **********************************************************************/
 
+#include <config.h>
 #include "drivermanager.h"
 
-static char const rcsid[]= "$RCSfile: SQLExecDirect.c,v $ $Revision: 1.10 $";
+static char const rcsid[]= "$RCSfile: SQLExecDirect.c,v $ $Revision: 1.11 $";
 
 SQLRETURN SQLExecDirectA( SQLHSTMT statement_handle,
            SQLCHAR *statement_text,
@@ -225,8 +229,8 @@ SQLRETURN SQLExecDirect( SQLHSTMT statement_handle,
         }
 
         sprintf( statement -> msg, "\n\t\tEntry:\
-            \n\t\t\tStatement = %p\
-            \n\t\t\tSQL = %s",
+\n\t\t\tStatement = %p\
+\n\t\t\tSQL = %s",
                 statement,
                 __string_with_length( s1, statement_text, text_length ));
 
@@ -280,7 +284,7 @@ SQLRETURN SQLExecDirect( SQLHSTMT statement_handle,
             statement -> state == STATE_S6 ||
             statement -> state == STATE_S7 )
 #else
-    if ( statement -> state == STATE_S6 ||
+    if (( statement -> state == STATE_S6 && statement -> eod == 0 ) ||
             statement -> state == STATE_S7 )
 #endif
     {
@@ -298,7 +302,10 @@ SQLRETURN SQLExecDirect( SQLHSTMT statement_handle,
     }
     else if ( statement -> state == STATE_S8 ||
             statement -> state == STATE_S9 ||
-            statement -> state == STATE_S10 )
+            statement -> state == STATE_S10 ||
+            statement -> state == STATE_S13 ||
+            statement -> state == STATE_S14 ||
+            statement -> state == STATE_S15 )
     {
         dm_log_write( __FILE__, 
                 __LINE__, 
@@ -335,6 +342,7 @@ SQLRETURN SQLExecDirect( SQLHSTMT statement_handle,
     if ( statement -> connection -> unicode_driver )
     {
         SQLWCHAR *s1;
+        int wlen;
 
 #ifdef NR_PROBE
         if ( !CHECK_SQLEXECDIRECTW( statement -> connection ) ||
@@ -369,7 +377,9 @@ SQLRETURN SQLExecDirect( SQLHSTMT statement_handle,
         }
 #endif
 
-        s1 = ansi_to_unicode_alloc( statement_text, text_length, statement -> connection );
+        s1 = ansi_to_unicode_alloc( statement_text, text_length, statement -> connection, &wlen );
+
+        text_length = wlen;
 
         ret = SQLEXECDIRECTW( statement -> connection,
                 statement -> driver_stmt,
@@ -431,7 +441,7 @@ SQLRETURN SQLExecDirect( SQLHSTMT statement_handle,
 
         if ( ret == SQL_SUCCESS_WITH_INFO )
         {
-            function_return_ex( SQL_HANDLE_STMT, statement, ret, TRUE );
+            function_return_ex( IGNORE_THREAD, statement, ret, TRUE );
         }
 
         local_ret = SQLNUMRESULTCOLS( statement -> connection,
@@ -475,6 +485,12 @@ SQLRETURN SQLExecDirect( SQLHSTMT statement_handle,
 
         statement -> prepared = 0;
     }
+    else if ( ret == SQL_PARAM_DATA_AVAILABLE )
+    {
+        statement -> interupted_func = SQL_API_SQLEXECDIRECT;
+        statement -> interupted_state = statement -> state;
+        statement -> state = STATE_S13;
+    }
     else if ( ret == SQL_STILL_EXECUTING )
     {
         statement -> interupted_func = SQL_API_SQLEXECDIRECT;
