@@ -4,7 +4,7 @@
  * (pharvey@codebydesign.com).
  *
  * Modified and extended by Nick Gorham
- * (nick@easysoft.com).
+ * (nick@lurcher.org).
  *
  * Any bugs or problems should be considered the fault of Nick and not
  * Peter.
@@ -27,9 +27,12 @@
  *
  **********************************************************************
  *
- * $Id: SQLForeignKeysW.c,v 1.8 2008/08/29 08:01:38 lurcher Exp $
+ * $Id: SQLForeignKeysW.c,v 1.9 2009/02/18 17:59:08 lurcher Exp $
  *
  * $Log: SQLForeignKeysW.c,v $
+ * Revision 1.9  2009/02/18 17:59:08  lurcher
+ * Shift to using config.h, the compile lines were making it hard to spot warnings
+ *
  * Revision 1.8  2008/08/29 08:01:38  lurcher
  * Alter the way W functions are passed to the driver
  *
@@ -79,6 +82,7 @@
  *
  **********************************************************************/
 
+#include <config.h>
 #include "drivermanager.h"
 
 static char const rcsid[]= "$RCSfile: SQLForeignKeysW.c,v $";
@@ -162,13 +166,13 @@ SQLRETURN SQLForeignKeysW(
     if ( log_info.log_flag )
     {
         sprintf( statement -> msg, "\n\t\tEntry:\
-            \n\t\t\tStatement = %p\
-            \n\t\t\tPK Catalog Name = %s\
-            \n\t\t\tPK Schema Name = %s\
-            \n\t\t\tPK Table Name = %s\
-            \n\t\t\tFK Catalog Name = %s\
-            \n\t\t\tFK Schema Name = %s\
-            \n\t\t\tFK Table Name = %s",
+\n\t\t\tStatement = %p\
+\n\t\t\tPK Catalog Name = %s\
+\n\t\t\tPK Schema Name = %s\
+\n\t\t\tPK Table Name = %s\
+\n\t\t\tFK Catalog Name = %s\
+\n\t\t\tFK Schema Name = %s\
+\n\t\t\tFK Table Name = %s",
                 statement,
                 __wstring_with_length( s1, szpk_catalog_name, cbpk_catalog_name ), 
                 __wstring_with_length( s2, szpk_schema_name, cbpk_schema_name ), 
@@ -227,7 +231,10 @@ SQLRETURN SQLForeignKeysW(
     }
     else if ( statement -> state == STATE_S8 ||
             statement -> state == STATE_S9 ||
-            statement -> state == STATE_S10 )
+            statement -> state == STATE_S10 ||
+            statement -> state == STATE_S13 ||
+            statement -> state == STATE_S14 ||
+            statement -> state == STATE_S15 )
     {
         dm_log_write( __FILE__, 
                 __LINE__, 
@@ -301,6 +308,7 @@ SQLRETURN SQLForeignKeysW(
     else
     {
         SQLCHAR *as1, *as2, *as3, *as4, *as5, *as6;
+        int clen;
 
         if ( !CHECK_SQLFOREIGNKEYS( statement -> connection ))
         {
@@ -317,12 +325,18 @@ SQLRETURN SQLForeignKeysW(
             return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
         }
 
-        as1 = (SQLCHAR*) unicode_to_ansi_alloc( szpk_catalog_name, cbpk_catalog_name, statement -> connection );
-        as2 = (SQLCHAR*) unicode_to_ansi_alloc( szpk_schema_name, cbpk_schema_name, statement -> connection );
-        as3 = (SQLCHAR*) unicode_to_ansi_alloc( szpk_table_name, cbpk_table_name, statement -> connection );
-        as4 = (SQLCHAR*) unicode_to_ansi_alloc( szfk_catalog_name, cbfk_catalog_name, statement -> connection );
-        as5 = (SQLCHAR*) unicode_to_ansi_alloc( szfk_schema_name, cbfk_schema_name, statement -> connection );
-        as6 = (SQLCHAR*) unicode_to_ansi_alloc( szfk_table_name, cbfk_table_name, statement -> connection );
+        as1 = (SQLCHAR*) unicode_to_ansi_alloc( szpk_catalog_name, cbpk_catalog_name, statement -> connection, &clen );
+        cbpk_catalog_name = clen;
+        as2 = (SQLCHAR*) unicode_to_ansi_alloc( szpk_schema_name, cbpk_schema_name, statement -> connection, &clen );
+        cbpk_schema_name = clen;
+        as3 = (SQLCHAR*) unicode_to_ansi_alloc( szpk_table_name, cbpk_table_name, statement -> connection, &clen );
+        cbpk_table_name = clen;
+        as4 = (SQLCHAR*) unicode_to_ansi_alloc( szfk_catalog_name, cbfk_catalog_name, statement -> connection, &clen );
+        cbfk_catalog_name = clen;
+        as5 = (SQLCHAR*) unicode_to_ansi_alloc( szfk_schema_name, cbfk_schema_name, statement -> connection, &clen );
+        cbfk_schema_name = clen;
+        as6 = (SQLCHAR*) unicode_to_ansi_alloc( szfk_table_name, cbfk_table_name, statement -> connection, &clen );
+        cbfk_table_name = clen;
 
         ret = SQLFOREIGNKEYS( statement -> connection ,
                 statement -> driver_stmt,
@@ -361,7 +375,7 @@ SQLRETURN SQLForeignKeysW(
 
         if ( ret == SQL_SUCCESS_WITH_INFO )
         {
-            function_return_ex( SQL_HANDLE_STMT, statement, ret, TRUE );
+            function_return_ex( IGNORE_THREAD, statement, ret, TRUE );
         }
 
         SQLNUMRESULTCOLS( statement -> connection,
