@@ -4,7 +4,7 @@
  * (pharvey@codebydesign.com).
  *
  * Modified and extended by Nick Gorham
- * (nick@easysoft.com).
+ * (nick@lurcher.org).
  *
  * Any bugs or problems should be considered the fault of Nick and not
  * Peter.
@@ -27,9 +27,15 @@
  *
  **********************************************************************
  *
- * $Id: SQLGetConnectAttr.c,v 1.13 2008/09/29 14:02:45 lurcher Exp $
+ * $Id: SQLGetConnectAttr.c,v 1.15 2009/02/18 17:59:08 lurcher Exp $
  *
  * $Log: SQLGetConnectAttr.c,v $
+ * Revision 1.15  2009/02/18 17:59:08  lurcher
+ * Shift to using config.h, the compile lines were making it hard to spot warnings
+ *
+ * Revision 1.14  2009/02/17 09:47:44  lurcher
+ * Clear up a number of bugs
+ *
  * Revision 1.13  2008/09/29 14:02:45  lurcher
  * Fix missing dlfcn group option
  *
@@ -155,9 +161,10 @@
  *
  **********************************************************************/
 
+#include <config.h>
 #include "drivermanager.h"
 
-static char const rcsid[]= "$RCSfile: SQLGetConnectAttr.c,v $ $Revision: 1.13 $";
+static char const rcsid[]= "$RCSfile: SQLGetConnectAttr.c,v $ $Revision: 1.15 $";
 
 SQLRETURN SQLGetConnectAttrA( SQLHDBC connection_handle,
            SQLINTEGER attribute,
@@ -197,7 +204,7 @@ SQLRETURN SQLGetConnectAttr( SQLHDBC connection_handle,
             }
             else
             {
-                *((SQLINTEGER*)value) = SQL_OPT_TRACE_ON;
+                *((SQLINTEGER*)value) = SQL_OPT_TRACE_OFF;
             }
         }
 
@@ -270,11 +277,11 @@ SQLRETURN SQLGetConnectAttr( SQLHDBC connection_handle,
     if ( log_info.log_flag )
     {
         sprintf( connection -> msg, "\n\t\tEntry:\
-            \n\t\t\tConnection = %p\
-            \n\t\t\tAttribute = %s\
-            \n\t\t\tValue = %p\
-            \n\t\t\tBuffer Length = %d\
-            \n\t\t\tStrLen = %p",
+\n\t\t\tConnection = %p\
+\n\t\t\tAttribute = %s\
+\n\t\t\tValue = %p\
+\n\t\t\tBuffer Length = %d\
+\n\t\t\tStrLen = %p",
                 connection,
                 __con_attr_as_string( s1, attribute ),
                 value, 
@@ -316,12 +323,6 @@ SQLRETURN SQLGetConnectAttr( SQLHDBC connection_handle,
           case SQL_ATTR_TRACE:
           case SQL_ATTR_TRACEFILE:
           case SQL_ATTR_ASYNC_ENABLE:
-          case SQL_ATTR_AUTO_IPD:
-          case SQL_ATTR_CONNECTION_TIMEOUT:
-          case SQL_ATTR_METADATA_ID:
-          case SQL_ATTR_PACKET_SIZE:
-          case SQL_ATTR_QUIET_MODE:
-          case SQL_ATTR_TXN_ISOLATION:
             break;
 
           default:
@@ -387,7 +388,7 @@ SQLRETURN SQLGetConnectAttr( SQLHDBC connection_handle,
         break;
 
       case SQL_ATTR_ODBC_CURSORS:
-        *((SQLINTEGER*)value) = connection -> cursors;
+        *((SQLULEN*)value) = connection -> cursors;
         type = 1;
         break;
 
@@ -411,7 +412,7 @@ SQLRETURN SQLGetConnectAttr( SQLHDBC connection_handle,
         }
         else
         {
-            *((SQLINTEGER*)value) = connection -> async_enable;
+            *((SQLULEN*)value) = connection -> async_enable;
             type = 1;
         }
         break;
@@ -568,7 +569,7 @@ SQLRETURN SQLGetConnectAttr( SQLHDBC connection_handle,
         {
             if ( !CHECK_SQLGETCONNECTATTRW( connection ))
             {
-                if ( ret = CHECK_SQLGETCONNECTOPTIONW( connection ))
+                if (( ret = CHECK_SQLGETCONNECTOPTIONW( connection )))
                 {
                     SQLWCHAR *s1 = NULL;
 
@@ -616,7 +617,7 @@ SQLRETURN SQLGetConnectAttr( SQLHDBC connection_handle,
                       case SQL_ATTR_TRANSLATE_LIB:
                         if ( SQL_SUCCEEDED( ret ) && value && s1 )
                         {
-                            unicode_to_ansi_copy( value, buffer_length, s1, SQL_NTS, connection );
+                            unicode_to_ansi_copy( value, buffer_length, s1, SQL_NTS, connection, NULL );
                         }
 						if ( SQL_SUCCEEDED( ret ) && string_length ) 
 						{
@@ -675,7 +676,7 @@ SQLRETURN SQLGetConnectAttr( SQLHDBC connection_handle,
                   case SQL_ATTR_TRANSLATE_LIB:
                     if ( SQL_SUCCEEDED( ret ) && value && s1 )
                     {
-                        unicode_to_ansi_copy( value, buffer_length, s1, SQL_NTS, connection );
+                        unicode_to_ansi_copy( value, buffer_length, s1, SQL_NTS, connection, NULL );
                     }
 					if ( SQL_SUCCEEDED( ret ) && string_length ) 
 					{
