@@ -4,7 +4,7 @@
  * (pharvey@codebydesign.com).
  *
  * Modified and extended by Nick Gorham
- * (nick@easysoft.com).
+ * (nick@lurcher.org).
  *
  * Any bugs or problems should be considered the fault of Nick and not
  * Peter.
@@ -27,9 +27,15 @@
  *
  **********************************************************************
  *
- * $Id: SQLGetConnectOption.c,v 1.7 2008/09/29 14:02:45 lurcher Exp $
+ * $Id: SQLGetConnectOption.c,v 1.9 2009/02/18 17:59:08 lurcher Exp $
  *
  * $Log: SQLGetConnectOption.c,v $
+ * Revision 1.9  2009/02/18 17:59:08  lurcher
+ * Shift to using config.h, the compile lines were making it hard to spot warnings
+ *
+ * Revision 1.8  2009/02/17 09:47:44  lurcher
+ * Clear up a number of bugs
+ *
  * Revision 1.7  2008/09/29 14:02:45  lurcher
  * Fix missing dlfcn group option
  *
@@ -135,9 +141,10 @@
  *
  **********************************************************************/
 
+#include <config.h>
 #include "drivermanager.h"
 
-static char const rcsid[]= "$RCSfile: SQLGetConnectOption.c,v $ $Revision: 1.7 $";
+static char const rcsid[]= "$RCSfile: SQLGetConnectOption.c,v $ $Revision: 1.9 $";
 
 SQLRETURN SQLGetConnectOptionA( SQLHDBC connection_handle,
            SQLUSMALLINT option,
@@ -211,9 +218,9 @@ SQLRETURN SQLGetConnectOption( SQLHDBC connection_handle,
     if ( log_info.log_flag )
     {
         sprintf( connection -> msg, "\n\t\tEntry:\
-            \n\t\t\tConnection = %p\
-            \n\t\t\tOption = %s\
-            \n\t\t\tValue = %p",
+\n\t\t\tConnection = %p\
+\n\t\t\tOption = %s\
+\n\t\t\tValue = %p",
                 connection,
                 __con_attr_as_string( s1, option ),
                 value );
@@ -302,6 +309,21 @@ SQLRETURN SQLGetConnectOption( SQLHDBC connection_handle,
         type = 1;
         break;
 
+      case SQL_ATTR_LOGIN_TIMEOUT:
+        /*
+         * if connected, call the driver
+         */
+        if ( connection -> state != STATE_C2 )
+        {
+            type = 0;
+        }
+        else
+        {
+            *((SQLINTEGER*)value) = connection -> login_timeout;
+            type = 1;
+        }
+        break;
+
       default:
         break;
     }
@@ -336,7 +358,7 @@ SQLRETURN SQLGetConnectOption( SQLHDBC connection_handle,
         {
             SQLWCHAR *s1 = NULL;
 
-            if ( ret = CHECK_SQLGETCONNECTOPTIONW( connection ))
+            if (( ret = CHECK_SQLGETCONNECTOPTIONW( connection )))
             {
                 switch( option )
                 {
@@ -368,7 +390,7 @@ SQLRETURN SQLGetConnectOption( SQLHDBC connection_handle,
                   case SQL_ATTR_TRANSLATE_LIB:
                     if ( SQL_SUCCEEDED( ret ) && value && s1 )
                     {
-                        unicode_to_ansi_copy( value, 1024, s1, SQL_NTS, connection );
+                        unicode_to_ansi_copy( value, 1024, s1, SQL_NTS, connection, NULL );
                     }
                     break;
                 }
@@ -414,7 +436,7 @@ SQLRETURN SQLGetConnectOption( SQLHDBC connection_handle,
 
                 if ( ptr != value && SQL_SUCCEEDED( ret ))
                 {
-                    unicode_to_ansi_copy( value, 1024, ptr, SQL_NTS, connection );
+                    unicode_to_ansi_copy( value, 1024, ptr, SQL_NTS, connection, NULL );
 
                     /*
                      * are we still here ? good
