@@ -172,7 +172,7 @@ SQLRETURN SQLGetCursorNameW( SQLHSTMT statement_handle,
 
     thread_protect( SQL_HANDLE_STMT, statement );
 
-    if ( !buffer_length < 0 )
+    if ( buffer_length < 0 )
     {
         dm_log_write( __FILE__, 
                 __LINE__, 
@@ -195,7 +195,10 @@ SQLRETURN SQLGetCursorNameW( SQLHSTMT statement_handle,
             statement -> state == STATE_S9 ||
             statement -> state == STATE_S10 ||
             statement -> state == STATE_S11 ||
-            statement -> state == STATE_S12 )
+            statement -> state == STATE_S12 ||
+            statement -> state == STATE_S13 ||
+            statement -> state == STATE_S14 ||
+            statement -> state == STATE_S15 )
     {
         dm_log_write( __FILE__, 
                 __LINE__, 
@@ -266,7 +269,7 @@ SQLRETURN SQLGetCursorNameW( SQLHSTMT statement_handle,
 
         if ( SQL_SUCCEEDED( ret ) && cursor_name && as1 )
         {
-            ansi_to_unicode_copy( cursor_name, (char*) as1, SQL_NTS, statement -> connection );
+            ansi_to_unicode_copy( cursor_name, (char*) as1, SQL_NTS, statement -> connection, NULL );
         }
 
         if ( as1 )
@@ -281,7 +284,7 @@ SQLRETURN SQLGetCursorNameW( SQLHSTMT statement_handle,
                 "\n\t\tExit:[%s]\
                 \n\t\t\tCursor Name = %s",
                     __get_return_status( ret, s1 ),
-                    __sdata_as_string( s1, SQL_CHAR, 
+                    __sdata_as_string( s1, SQL_WCHAR, 
                         name_length, cursor_name ));
 
         dm_log_write( __FILE__, 
