@@ -4,7 +4,7 @@
  * (pharvey@codebydesign.com).
  *
  * Modified and extended by Nick Gorham
- * (nick@easysoft.com).
+ * (nick@lurcher.org).
  *
  * Any bugs or problems should be considered the fault of Nick and not
  * Peter.
@@ -27,9 +27,15 @@
  *
  **********************************************************************
  *
- * $Id: SQLGetDiagRecW.c,v 1.9 2007/11/26 11:37:23 lurcher Exp $
+ * $Id: SQLGetDiagRecW.c,v 1.11 2009/02/18 17:59:08 lurcher Exp $
  *
  * $Log: SQLGetDiagRecW.c,v $
+ * Revision 1.11  2009/02/18 17:59:08  lurcher
+ * Shift to using config.h, the compile lines were making it hard to spot warnings
+ *
+ * Revision 1.10  2009/02/04 09:30:02  lurcher
+ * Fix some SQLINTEGER/SQLLEN conflicts
+ *
  * Revision 1.9  2007/11/26 11:37:23  lurcher
  * Sync up before tag
  *
@@ -87,6 +93,7 @@
  *
  **********************************************************************/
 
+#include <config.h>
 #include "drivermanager.h"
 
 static char const rcsid[]= "$RCSfile: SQLGetDiagRecW.c,v $";
@@ -105,7 +112,7 @@ static SQLRETURN extract_sql_error_rec_w( EHEAD *head,
     {
         SQLWCHAR *tmp;
 
-        tmp = ansi_to_unicode_alloc((SQLCHAR*) "00000", SQL_NTS, __get_connection( head ));
+        tmp = ansi_to_unicode_alloc((SQLCHAR*) "00000", SQL_NTS, __get_connection( head ), NULL );
         wide_strcpy( sqlstate, tmp );
         free( tmp );
     }
@@ -121,6 +128,11 @@ static SQLRETURN extract_sql_error_rec_w( EHEAD *head,
             rec_number --;
         }
 
+		if ( !ptr ) 
+		{
+	    	return SQL_NO_DATA;
+		}
+
         if ( sqlstate )
         {
             wide_strcpy( sqlstate, ptr -> sqlstate );
@@ -231,7 +243,7 @@ static SQLRETURN extract_sql_error_rec_w( EHEAD *head,
                 {
                     if ( as1 )
                     {
-                        ansi_to_unicode_copy( sqlstate,(char*) as1, SQL_NTS, __get_connection( head ));
+                        ansi_to_unicode_copy( sqlstate,(char*) as1, SQL_NTS, __get_connection( head ), NULL );
                         __map_error_state_w( sqlstate, __get_version( head ));
                     }
                 }
@@ -239,7 +251,7 @@ static SQLRETURN extract_sql_error_rec_w( EHEAD *head,
                 {
                     if ( as2 )
                     {
-                        ansi_to_unicode_copy( message_text,(char*) as2, SQL_NTS, __get_connection( head ));
+                        ansi_to_unicode_copy( message_text,(char*) as2, SQL_NTS, __get_connection( head ), NULL );
                     }
                 }
             }
@@ -258,6 +270,11 @@ static SQLRETURN extract_sql_error_rec_w( EHEAD *head,
                 rec_number --;
             }
 
+			if ( !ptr ) 
+			{
+	    		return SQL_NO_DATA;
+			}
+
             if ( sqlstate )
             {
                 wide_strcpy( sqlstate, ptr -> sqlstate );
@@ -322,6 +339,7 @@ SQLRETURN SQLGetDiagRecW( SQLSMALLINT handle_type,
     SQLRETURN ret;
     SQLCHAR s0[ 32 ], s1[ 100 + LOG_MESSAGE_LEN ];
     SQLCHAR s2[ 100 + LOG_MESSAGE_LEN ];
+    SQLCHAR s3[ 100 + LOG_MESSAGE_LEN ];
 
     if ( rec_number < 1 )
     {
@@ -349,13 +367,13 @@ SQLRETURN SQLGetDiagRecW( SQLSMALLINT handle_type,
         {
             sprintf( environment -> msg, 
                 "\n\t\tEntry:\
-                \n\t\t\tEnvironment = %p\
-                \n\t\t\tRec Number = %d\
-                \n\t\t\tSQLState = %p\
-                \n\t\t\tNative = %p\
-                \n\t\t\tMessage Text = %p\
-                \n\t\t\tBuffer Length = %d\
-                \n\t\t\tText Len Ptr = %p",
+\n\t\t\tEnvironment = %p\
+\n\t\t\tRec Number = %d\
+\n\t\t\tSQLState = %p\
+\n\t\t\tNative = %p\
+\n\t\t\tMessage Text = %p\
+\n\t\t\tBuffer Length = %d\
+\n\t\t\tText Len Ptr = %p",
                     environment,
                     rec_number,
                     sqlstate,
@@ -387,17 +405,22 @@ SQLRETURN SQLGetDiagRecW( SQLSMALLINT handle_type,
 
                 sprintf( environment -> msg, 
                     "\n\t\tExit:[%s]\
-                    \n\t\t\tSQLState = %s\
-                    \n\t\t\tNative = %s\
-                    \n\t\t\tMessage Text = %s",
+\n\t\t\tSQLState = %s\
+\n\t\t\tNative = %s\
+\n\t\t\tMessage Text = %s",
                         __get_return_status( ret, s2 ),
-                        ts1 = unicode_to_ansi_alloc( sqlstate, SQL_NTS, NULL ),
+                        __sdata_as_string( s3, SQL_CHAR,
+				NULL, ts1 = unicode_to_ansi_alloc( sqlstate, SQL_NTS, NULL, NULL  )),
                         __iptr_as_string( s0, native ),
                         __sdata_as_string( s1, SQL_CHAR, 
-                            text_length_ptr, ts2 = unicode_to_ansi_alloc( message_text, SQL_NTS, NULL )));
+                            text_length_ptr, ts2 = unicode_to_ansi_alloc( message_text, SQL_NTS, NULL, NULL  )));
 
-                free( ts1 );
-                free( ts2 );
+		if ( ts1 ) {
+                	free( ts1 );
+		}
+		if ( ts2 ) {
+                	free( ts2 );
+		}
             }
             else
             {
@@ -472,13 +495,13 @@ SQLRETURN SQLGetDiagRecW( SQLSMALLINT handle_type,
         {
             sprintf( connection -> msg, 
                 "\n\t\tEntry:\
-                \n\t\t\tConnection = %p\
-                \n\t\t\tRec Number = %d\
-                \n\t\t\tSQLState = %p\
-                \n\t\t\tNative = %p\
-                \n\t\t\tMessage Text = %p\
-                \n\t\t\tBuffer Length = %d\
-                \n\t\t\tText Len Ptr = %p",
+\n\t\t\tConnection = %p\
+\n\t\t\tRec Number = %d\
+\n\t\t\tSQLState = %p\
+\n\t\t\tNative = %p\
+\n\t\t\tMessage Text = %p\
+\n\t\t\tBuffer Length = %d\
+\n\t\t\tText Len Ptr = %p",
                     connection,
                     rec_number,
                     sqlstate,
@@ -510,17 +533,22 @@ SQLRETURN SQLGetDiagRecW( SQLSMALLINT handle_type,
 
                 sprintf( connection -> msg, 
                     "\n\t\tExit:[%s]\
-                    \n\t\t\tSQLState = %s\
-                    \n\t\t\tNative = %s\
-                    \n\t\t\tMessage Text = %s",
+\n\t\t\tSQLState = %s\
+\n\t\t\tNative = %s\
+\n\t\t\tMessage Text = %s",
                         __get_return_status( ret, s2 ),
-                        ts1 = unicode_to_ansi_alloc( sqlstate, SQL_NTS, connection ),
+                        __sdata_as_string( s3, SQL_CHAR,
+				NULL, ts1 = unicode_to_ansi_alloc( sqlstate, SQL_NTS, connection, NULL  )),
                         __iptr_as_string( s0, native ),
                         __sdata_as_string( s1, SQL_CHAR, 
-                            text_length_ptr, ts2 = unicode_to_ansi_alloc( message_text, SQL_NTS, connection )));
+                            	text_length_ptr, ts2 = unicode_to_ansi_alloc( message_text, SQL_NTS, connection, NULL  )));
 
-                free( ts1 );
-                free( ts2 );
+		if ( ts1 ) {
+                	free( ts1 );
+		}
+		if ( ts2 ) {
+                	free( ts2 );
+		}
             }
             else
             {
@@ -595,13 +623,13 @@ SQLRETURN SQLGetDiagRecW( SQLSMALLINT handle_type,
         {
             sprintf( statement -> msg, 
                 "\n\t\tEntry:\
-                \n\t\t\tStatement = %p\
-                \n\t\t\tRec Number = %d\
-                \n\t\t\tSQLState = %p\
-                \n\t\t\tNative = %p\
-                \n\t\t\tMessage Text = %p\
-                \n\t\t\tBuffer Length = %d\
-                \n\t\t\tText Len Ptr = %p",
+\n\t\t\tStatement = %p\
+\n\t\t\tRec Number = %d\
+\n\t\t\tSQLState = %p\
+\n\t\t\tNative = %p\
+\n\t\t\tMessage Text = %p\
+\n\t\t\tBuffer Length = %d\
+\n\t\t\tText Len Ptr = %p",
                     statement,
                     rec_number,
                     sqlstate,
@@ -633,17 +661,22 @@ SQLRETURN SQLGetDiagRecW( SQLSMALLINT handle_type,
 
                 sprintf( statement -> msg, 
                     "\n\t\tExit:[%s]\
-                    \n\t\t\tSQLState = %s\
-                    \n\t\t\tNative = %s\
-                    \n\t\t\tMessage Text = %s",
+\n\t\t\tSQLState = %s\
+\n\t\t\tNative = %s\
+\n\t\t\tMessage Text = %s",
                         __get_return_status( ret, s2 ),
-                        ts1 = unicode_to_ansi_alloc( sqlstate, SQL_NTS, statement -> connection ),
+                        __sdata_as_string( s3, SQL_CHAR,
+				NULL, ts1 = unicode_to_ansi_alloc( sqlstate, SQL_NTS, statement -> connection, NULL  )),
                         __iptr_as_string( s0, native ),
                         __sdata_as_string( s1, SQL_CHAR, 
-                            text_length_ptr, ts2 = unicode_to_ansi_alloc( message_text, SQL_NTS, statement -> connection )));
+                            text_length_ptr, ts2 = unicode_to_ansi_alloc( message_text, SQL_NTS, statement -> connection, NULL  )));
 
-                free( ts1 );
-                free( ts2 );
+		if ( ts1 ) {
+                	free( ts1 );
+		}
+		if ( ts2 ) {
+                	free( ts2 );
+		}
             }
             else
             {
@@ -718,13 +751,13 @@ SQLRETURN SQLGetDiagRecW( SQLSMALLINT handle_type,
         {
             sprintf( descriptor -> msg, 
                 "\n\t\tEntry:\
-                \n\t\t\tDescriptor = %p\
-                \n\t\t\tRec Number = %d\
-                \n\t\t\tSQLState = %p\
-                \n\t\t\tNative = %p\
-                \n\t\t\tMessage Text = %p\
-                \n\t\t\tBuffer Length = %d\
-                \n\t\t\tText Len Ptr = %p",
+\n\t\t\tDescriptor = %p\
+\n\t\t\tRec Number = %d\
+\n\t\t\tSQLState = %p\
+\n\t\t\tNative = %p\
+\n\t\t\tMessage Text = %p\
+\n\t\t\tBuffer Length = %d\
+\n\t\t\tText Len Ptr = %p",
                     descriptor,
                     rec_number,
                     sqlstate,
@@ -756,17 +789,22 @@ SQLRETURN SQLGetDiagRecW( SQLSMALLINT handle_type,
 
                 sprintf( descriptor -> msg, 
                     "\n\t\tExit:[%s]\
-                    \n\t\t\tSQLState = %s\
-                    \n\t\t\tNative = %s\
-                    \n\t\t\tMessage Text = %s",
+\n\t\t\tSQLState = %s\
+\n\t\t\tNative = %s\
+\n\t\t\tMessage Text = %s",
                         __get_return_status( ret, s2 ),
-                        ts1 = unicode_to_ansi_alloc( sqlstate, SQL_NTS, descriptor -> connection ),
+                        __sdata_as_string( s3, SQL_CHAR,
+				NULL, ts1 = unicode_to_ansi_alloc( sqlstate, SQL_NTS, descriptor -> connection, NULL  )),
                         __iptr_as_string( s0, native ),
                         __sdata_as_string( s1, SQL_CHAR, 
-                            text_length_ptr, ts2 = unicode_to_ansi_alloc( message_text, SQL_NTS, descriptor -> connection )));
+                            text_length_ptr, ts2 = unicode_to_ansi_alloc( message_text, SQL_NTS, descriptor -> connection, NULL  )));
 
-                free( ts1 );
-                free( ts2 );
+		if ( ts1 ) {
+                	free( ts1 );
+		}
+		if ( ts2 ) {
+                	free( ts2 );
+		}
             }
             else
             {
