@@ -4,7 +4,7 @@
  * (pharvey@codebydesign.com).
  *
  * Modified and extended by Nick Gorham
- * (nick@easysoft.com).
+ * (nick@lurcher.org).
  *
  * Any bugs or problems should be considered the fault of Nick and not
  * Peter.
@@ -27,9 +27,12 @@
  *
  **********************************************************************
  *
- * $Id: SQLPrimaryKeys.c,v 1.6 2004/01/12 09:54:39 lurcher Exp $
+ * $Id: SQLPrimaryKeys.c,v 1.7 2009/02/18 17:59:08 lurcher Exp $
  *
  * $Log: SQLPrimaryKeys.c,v $
+ * Revision 1.7  2009/02/18 17:59:08  lurcher
+ * Shift to using config.h, the compile lines were making it hard to spot warnings
+ *
  * Revision 1.6  2004/01/12 09:54:39  lurcher
  *
  * Fix problem where STATE_S5 stops metadata calls
@@ -129,9 +132,10 @@
  *
  **********************************************************************/
 
+#include <config.h>
 #include "drivermanager.h"
 
-static char const rcsid[]= "$RCSfile: SQLPrimaryKeys.c,v $ $Revision: 1.6 $";
+static char const rcsid[]= "$RCSfile: SQLPrimaryKeys.c,v $ $Revision: 1.7 $";
 
 SQLRETURN SQLPrimaryKeysA(
     SQLHSTMT           statement_handle,
@@ -184,10 +188,10 @@ SQLRETURN SQLPrimaryKeys(
     if ( log_info.log_flag )
     {
         sprintf( statement -> msg, "\n\t\tEntry:\
-            \n\t\t\tStatement = %p\
-            \n\t\t\tCatalog Name = %s\
-            \n\t\t\tSchema Name = %s\
-            \n\t\t\tTable Type = %s", 
+\n\t\t\tStatement = %p\
+\n\t\t\tCatalog Name = %s\
+\n\t\t\tSchema Name = %s\
+\n\t\t\tTable Type = %s", 
                 statement,
                 __string_with_length( s1, sz_catalog_name, cb_catalog_name ), 
                 __string_with_length( s2, sz_schema_name, cb_schema_name ), 
@@ -246,7 +250,10 @@ SQLRETURN SQLPrimaryKeys(
     }
     else if ( statement -> state == STATE_S8 ||
             statement -> state == STATE_S9 ||
-            statement -> state == STATE_S10 )
+            statement -> state == STATE_S10 ||
+            statement -> state == STATE_S13 ||
+            statement -> state == STATE_S14 ||
+            statement -> state == STATE_S15 )
     {
         dm_log_write( __FILE__, 
                 __LINE__, 
@@ -280,6 +287,21 @@ SQLRETURN SQLPrimaryKeys(
         }
     }
 
+    if ( sz_table_name == NULL ) 
+    {
+        dm_log_write( __FILE__, 
+                __LINE__, 
+                LOG_INFO, 
+                LOG_INFO, 
+                "Error: HY009" );
+
+        __post_internal_error( &statement -> error,
+                ERROR_HY009, NULL,
+                statement -> connection -> environment -> requested_version );
+
+        return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
+    }
+
     /*
      * TO_DO Check the SQL_ATTR_METADATA_ID settings
      */
@@ -287,6 +309,7 @@ SQLRETURN SQLPrimaryKeys(
     if ( statement -> connection -> unicode_driver )
     {
         SQLWCHAR *s1, *s2, *s3;
+        int wlen;
 
         if ( !CHECK_SQLPRIMARYKEYSW( statement -> connection ))
         {
@@ -303,9 +326,12 @@ SQLRETURN SQLPrimaryKeys(
             return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
         }
 
-        s1 = ansi_to_unicode_alloc( sz_catalog_name, cb_catalog_name, statement -> connection );
-        s2 = ansi_to_unicode_alloc( sz_schema_name, cb_schema_name, statement -> connection );
-        s3 = ansi_to_unicode_alloc( sz_table_name, cb_table_name, statement -> connection );
+        s1 = ansi_to_unicode_alloc( sz_catalog_name, cb_catalog_name, statement -> connection, &wlen );
+        cb_catalog_name = wlen;
+        s2 = ansi_to_unicode_alloc( sz_schema_name, cb_schema_name, statement -> connection, &wlen );
+        cb_schema_name = wlen;
+        s3 = ansi_to_unicode_alloc( sz_table_name, cb_table_name, statement -> connection, &wlen );
+        cb_table_name = wlen;
 
         ret = SQLPRIMARYKEYSW( statement -> connection ,
                 statement -> driver_stmt,
@@ -364,7 +390,7 @@ SQLRETURN SQLPrimaryKeys(
 
         if ( ret == SQL_SUCCESS_WITH_INFO )
         {
-            function_return_ex( SQL_HANDLE_STMT, statement, ret, TRUE );
+            function_return_ex( IGNORE_THREAD, statement, ret, TRUE );
         }
 
         SQLNUMRESULTCOLS( statement -> connection,
