@@ -4,7 +4,7 @@
  * (pharvey@codebydesign.com).
  *
  * Modified and extended by Nick Gorham
- * (nick@easysoft.com).
+ * (nick@lurcher.org).
  *
  * Any bugs or problems should be considered the fault of Nick and not
  * Peter.
@@ -27,9 +27,12 @@
  *
  **********************************************************************
  *
- * $Id: SQLProcedures.c,v 1.6 2004/01/12 09:54:39 lurcher Exp $
+ * $Id: SQLProcedures.c,v 1.7 2009/02/18 17:59:08 lurcher Exp $
  *
  * $Log: SQLProcedures.c,v $
+ * Revision 1.7  2009/02/18 17:59:08  lurcher
+ * Shift to using config.h, the compile lines were making it hard to spot warnings
+ *
  * Revision 1.6  2004/01/12 09:54:39  lurcher
  *
  * Fix problem where STATE_S5 stops metadata calls
@@ -117,9 +120,10 @@
  *
  **********************************************************************/
 
+#include <config.h>
 #include "drivermanager.h"
 
-static char const rcsid[]= "$RCSfile: SQLProcedures.c,v $ $Revision: 1.6 $";
+static char const rcsid[]= "$RCSfile: SQLProcedures.c,v $ $Revision: 1.7 $";
 
 SQLRETURN SQLProceduresA(
     SQLHSTMT           statement_handle,
@@ -172,10 +176,10 @@ SQLRETURN SQLProcedures(
     if ( log_info.log_flag )
     {
         sprintf( statement -> msg, "\n\t\tEntry:\
-            \n\t\t\tStatement = %p\
-            \n\t\t\tCatalog Name = %s\
-            \n\t\t\tSchema Name = %s\
-            \n\t\t\tProc Name = %s",
+\n\t\t\tStatement = %p\
+\n\t\t\tCatalog Name = %s\
+\n\t\t\tSchema Name = %s\
+\n\t\t\tProc Name = %s",
                 statement,
                 __string_with_length( s1, sz_catalog_name, cb_catalog_name  ), 
                 __string_with_length( s2, sz_schema_name, cb_schema_name ), 
@@ -216,7 +220,7 @@ SQLRETURN SQLProcedures(
             statement -> state == STATE_S6 ||
             statement -> state == STATE_S7 )
 #else
-    if ( statement -> state == STATE_S6 ||
+    if (( statement -> state == STATE_S6 && statement -> eod == 0 ) ||
             statement -> state == STATE_S7 )
 #endif
     {
@@ -234,7 +238,10 @@ SQLRETURN SQLProcedures(
     }
     else if ( statement -> state == STATE_S8 ||
             statement -> state == STATE_S9 ||
-            statement -> state == STATE_S10 )
+            statement -> state == STATE_S10 ||
+            statement -> state == STATE_S13 ||
+            statement -> state == STATE_S14 ||
+            statement -> state == STATE_S15 )
     {
         dm_log_write( __FILE__, 
                 __LINE__, 
@@ -275,6 +282,7 @@ SQLRETURN SQLProcedures(
     if ( statement -> connection -> unicode_driver )
     {
         SQLWCHAR *s1, *s2, *s3;
+        int wlen;
 
         if ( !CHECK_SQLPROCEDURESW( statement -> connection ))
         {
@@ -291,9 +299,12 @@ SQLRETURN SQLProcedures(
             return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
         }
 
-        s1 = ansi_to_unicode_alloc( sz_catalog_name, cb_catalog_name, statement -> connection );
-        s2 = ansi_to_unicode_alloc( sz_schema_name, cb_schema_name, statement -> connection );
-        s3 = ansi_to_unicode_alloc( sz_proc_name, cb_proc_name, statement -> connection );
+        s1 = ansi_to_unicode_alloc( sz_catalog_name, cb_catalog_name, statement -> connection, &wlen );
+        cb_catalog_name = wlen;
+        s2 = ansi_to_unicode_alloc( sz_schema_name, cb_schema_name, statement -> connection, &wlen );
+        cb_schema_name = wlen;
+        s3 = ansi_to_unicode_alloc( sz_proc_name, cb_proc_name, statement -> connection, &wlen );
+        cb_proc_name = wlen;
 
         ret = SQLPROCEDURESW( statement -> connection ,
                 statement -> driver_stmt,
