@@ -4,7 +4,7 @@
  * (pharvey@codebydesign.com).
  *
  * Modified and extended by Nick Gorham
- * (nick@easysoft.com).
+ * (nick@lurcher.org).
  *
  * Any bugs or problems should be considered the fault of Nick and not
  * Peter.
@@ -27,9 +27,12 @@
  *
  **********************************************************************
  *
- * $Id: SQLProceduresW.c,v 1.8 2008/08/29 08:01:39 lurcher Exp $
+ * $Id: SQLProceduresW.c,v 1.9 2009/02/18 17:59:08 lurcher Exp $
  *
  * $Log: SQLProceduresW.c,v $
+ * Revision 1.9  2009/02/18 17:59:08  lurcher
+ * Shift to using config.h, the compile lines were making it hard to spot warnings
+ *
  * Revision 1.8  2008/08/29 08:01:39  lurcher
  * Alter the way W functions are passed to the driver
  *
@@ -79,9 +82,10 @@
  *
  **********************************************************************/
 
+#include <config.h>
 #include "drivermanager.h"
 
-static char const rcsid[]= "$RCSfile: SQLProceduresW.c,v $ $Revision: 1.8 $";
+static char const rcsid[]= "$RCSfile: SQLProceduresW.c,v $ $Revision: 1.9 $";
 
 SQLRETURN SQLProceduresW(
     SQLHSTMT           statement_handle,
@@ -149,10 +153,10 @@ SQLRETURN SQLProceduresW(
     if ( log_info.log_flag )
     {
         sprintf( statement -> msg, "\n\t\tEntry:\
-            \n\t\t\tStatement = %p\
-            \n\t\t\tCatalog Name = %s\
-            \n\t\t\tSchema Name = %s\
-            \n\t\t\tProc Name = %s",
+\n\t\t\tStatement = %p\
+\n\t\t\tCatalog Name = %s\
+\n\t\t\tSchema Name = %s\
+\n\t\t\tProc Name = %s",
                 statement,
                 __wstring_with_length( s1, sz_catalog_name, cb_catalog_name  ), 
                 __wstring_with_length( s2, sz_schema_name, cb_schema_name ), 
@@ -193,7 +197,7 @@ SQLRETURN SQLProceduresW(
             statement -> state == STATE_S6 ||
             statement -> state == STATE_S7 )
 #else
-    if ( statement -> state == STATE_S6 ||
+    if (( statement -> state == STATE_S6 && statement -> eod == 0 ) ||
             statement -> state == STATE_S7 )
 #endif
     {
@@ -211,7 +215,10 @@ SQLRETURN SQLProceduresW(
     }
     else if ( statement -> state == STATE_S8 ||
             statement -> state == STATE_S9 ||
-            statement -> state == STATE_S10 )
+            statement -> state == STATE_S10 ||
+            statement -> state == STATE_S13 ||
+            statement -> state == STATE_S14 ||
+            statement -> state == STATE_S15 )
     {
         dm_log_write( __FILE__, 
                 __LINE__, 
@@ -279,6 +286,7 @@ SQLRETURN SQLProceduresW(
     else
     {
         SQLCHAR *as1, *as2, *as3;
+        int clen;
 
         if ( !CHECK_SQLPROCEDURES( statement -> connection ))
         {
@@ -295,9 +303,12 @@ SQLRETURN SQLProceduresW(
             return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
         }
 
-        as1 = (SQLCHAR*) unicode_to_ansi_alloc( sz_catalog_name, cb_catalog_name, statement -> connection );
-        as2 = (SQLCHAR*) unicode_to_ansi_alloc( sz_schema_name, cb_schema_name, statement -> connection );
-        as3 = (SQLCHAR*) unicode_to_ansi_alloc( sz_proc_name, cb_proc_name, statement -> connection );
+        as1 = (SQLCHAR*) unicode_to_ansi_alloc( sz_catalog_name, cb_catalog_name, statement -> connection, &clen );
+        cb_catalog_name = clen;
+        as2 = (SQLCHAR*) unicode_to_ansi_alloc( sz_schema_name, cb_schema_name, statement -> connection, &clen );
+        cb_schema_name = clen;
+        as3 = (SQLCHAR*) unicode_to_ansi_alloc( sz_proc_name, cb_proc_name, statement -> connection, &clen );
+        cb_proc_name = clen;
 
         ret = SQLPROCEDURES( statement -> connection ,
                 statement -> driver_stmt,
