@@ -4,7 +4,7 @@
  * (pharvey@codebydesign.com).
  *
  * Modified and extended by Nick Gorham
- * (nick@easysoft.com).
+ * (nick@lurcher.org).
  *
  * Any bugs or problems should be considered the fault of Nick and not
  * Peter.
@@ -27,9 +27,12 @@
  *
  **********************************************************************
  *
- * $Id: SQLSetConnectAttrW.c,v 1.13 2008/08/29 08:01:39 lurcher Exp $
+ * $Id: SQLSetConnectAttrW.c,v 1.14 2009/02/18 17:59:08 lurcher Exp $
  *
  * $Log: SQLSetConnectAttrW.c,v $
+ * Revision 1.14  2009/02/18 17:59:08  lurcher
+ * Shift to using config.h, the compile lines were making it hard to spot warnings
+ *
  * Revision 1.13  2008/08/29 08:01:39  lurcher
  * Alter the way W functions are passed to the driver
  *
@@ -119,6 +122,7 @@
  *
  **********************************************************************/
 
+#include <config.h>
 #include "drivermanager.h"
 
 static char const rcsid[]= "$RCSfile: SQLSetConnectAttrW.c,v $";
@@ -139,6 +143,30 @@ SQLRETURN SQLSetConnectAttrW( SQLHDBC connection_handle,
 
     if ( attribute == SQL_ATTR_TRACE )
     {
+        if ((SQLLEN) value != SQL_OPT_TRACE_OFF && 
+            (SQLLEN) value != SQL_OPT_TRACE_ON ) 
+        {
+            if ( __validate_dbc( connection ))
+            {
+                thread_protect( SQL_HANDLE_DBC, connection );
+                dm_log_write( __FILE__, 
+                        __LINE__, 
+                        LOG_INFO, 
+                        LOG_INFO, 
+                        "Error: HY024" );
+        
+                __post_internal_error( &connection -> error,
+                    ERROR_HY024, NULL,
+                    connection -> environment -> requested_version );
+        
+                return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+            }
+            else 
+            {
+                return SQL_INVALID_HANDLE;
+            }
+        }
+
         if ((SQLLEN) value == SQL_OPT_TRACE_OFF )
         {
             char force_string[ 30 ];
@@ -185,11 +213,58 @@ SQLRETURN SQLSetConnectAttrW( SQLHDBC connection_handle,
     {
         if ( value )
         {
-            if ( log_info.log_file_name )
+            if (((SQLWCHAR*)value)[ 0 ] == 0 ) 
             {
-                free( log_info.log_file_name );
+                if ( __validate_dbc( connection ))
+                {
+                    thread_protect( SQL_HANDLE_DBC, connection );
+                    dm_log_write( __FILE__, 
+                            __LINE__, 
+                            LOG_INFO, 
+                            LOG_INFO, 
+                            "Error: HY024" );
+            
+                    __post_internal_error( &connection -> error,
+                        ERROR_HY024, NULL,
+                        connection -> environment -> requested_version );
+            
+                    return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+                }
+                else 
+                {
+                    return SQL_INVALID_HANDLE;
+                }
+            }
+            else 
+            {
+                if ( log_info.log_file_name )
+                {
+                    free( log_info.log_file_name );
+                }
+                log_info.log_file_name = unicode_to_ansi_alloc((SQLWCHAR *) value, SQL_NTS, connection, NULL );
+            }
+        }
+        else 
+        {
+            if ( __validate_dbc( connection ))
+            {
+                thread_protect( SQL_HANDLE_DBC, connection );
+                dm_log_write( __FILE__, 
+                        __LINE__, 
+                        LOG_INFO, 
+                        LOG_INFO, 
+                        "Error: HY009" );
+        
+                __post_internal_error( &connection -> error,
+                    ERROR_HY009, NULL,
+                    connection -> environment -> requested_version );
+        
+                return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+            }
+            else 
+            {
+                return SQL_INVALID_HANDLE;
             }
-            log_info.log_file_name = unicode_to_ansi_alloc( value, SQL_NTS, connection );
         }
         return SQL_SUCCESS;
     }
@@ -244,14 +319,14 @@ SQLRETURN SQLSetConnectAttrW( SQLHDBC connection_handle,
     if ( log_info.log_flag )
     {
         sprintf( connection -> msg, "\n\t\tEntry:\
-            \n\t\t\tConnection = %p\
-            \n\t\t\tAttribute = %s\
-            \n\t\t\tValue = %p\
-            \n\t\t\tStrLen = %d",
+\n\t\t\tConnection = %p\
+\n\t\t\tAttribute = %s\
+\n\t\t\tValue = %p\
+\n\t\t\tStrLen = %d",
                 connection,
                 __con_attr_as_string( s1, attribute ),
                 value, 
-                string_length );
+                (int)string_length );
 
         dm_log_write( __FILE__, 
                 __LINE__, 
@@ -328,6 +403,98 @@ SQLRETURN SQLSetConnectAttrW( SQLHDBC connection_handle,
         }
     }
 
+    /*
+     * is it a legitimate value
+     */
+    ret = dm_check_connection_attrs( connection, attribute, value );
+
+    if ( ret != SQL_SUCCESS ) 
+    {
+        dm_log_write( __FILE__, 
+                    __LINE__, 
+                    LOG_INFO, 
+                    LOG_INFO, 
+                    "Error: HY024" );
+
+        __post_internal_error( &connection -> error,
+                ERROR_HY024, NULL,
+                connection -> environment -> requested_version );
+
+        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+    }
+
+    /*
+     * is it a connection attribute or statement, check state of any active connections
+     */
+
+    switch( attribute ) 
+    {
+      	case SQL_ATTR_CONCURRENCY:
+	  	case SQL_BIND_TYPE:
+      	case SQL_ATTR_CURSOR_SCROLLABLE:
+      	case SQL_ATTR_CURSOR_SENSITIVITY:
+      	case SQL_ATTR_CURSOR_TYPE:
+      	case SQL_ATTR_MAX_LENGTH:
+      	case SQL_MAX_ROWS:
+      	case SQL_ATTR_KEYSET_SIZE:
+      	case SQL_ROWSET_SIZE: 
+      	case SQL_ATTR_NOSCAN:
+      	case SQL_ATTR_QUERY_TIMEOUT:
+      	case SQL_ATTR_RETRIEVE_DATA:
+      	case SQL_ATTR_SIMULATE_CURSOR:
+      	case SQL_ATTR_USE_BOOKMARKS:
+            if( __check_stmt_from_dbc_v( connection, 8, STATE_S8, STATE_S9, STATE_S10, STATE_S11, STATE_S12, STATE_S13, STATE_S14, STATE_S15 )) {
+
+                dm_log_write( __FILE__, 
+                        __LINE__, 
+                        LOG_INFO, 
+                        LOG_INFO, 
+                        "Error: 24000" );
+
+                __post_internal_error( &connection -> error,
+                        ERROR_24000, NULL,
+                        connection -> environment -> requested_version );
+
+                return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+            }
+            break;
+
+        default:
+            if ( attribute == SQL_ATTR_CURRENT_CATALOG ) 
+            {
+                if( __check_stmt_from_dbc_v( connection, 3, STATE_S5, STATE_S6, STATE_S7 )) {
+
+                    dm_log_write( __FILE__, 
+                            __LINE__, 
+                            LOG_INFO, 
+                            LOG_INFO, 
+                            "Error: 24000" );
+
+                    __post_internal_error( &connection -> error,
+                            ERROR_24000, NULL,
+                            connection -> environment -> requested_version );
+
+                    return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+                }
+            }
+
+            if( __check_stmt_from_dbc_v( connection, 8, STATE_S8, STATE_S9, STATE_S10, STATE_S11, STATE_S12, STATE_S13, STATE_S14, STATE_S15 )) {
+
+                dm_log_write( __FILE__, 
+                        __LINE__, 
+                        LOG_INFO, 
+                        LOG_INFO, 
+                        "Error: HY010" );
+
+                __post_internal_error( &connection -> error,
+                        ERROR_HY010, NULL,
+                        connection -> environment -> requested_version );
+
+                return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+            }
+            break;
+    }
+
     /*
      * is it something overridden
      */
@@ -335,7 +502,7 @@ SQLRETURN SQLSetConnectAttrW( SQLHDBC connection_handle,
     value = __attr_override_wide( connection, SQL_HANDLE_DBC, attribute, value, &string_length, buffer );
 
     /*
-     * we need to save this even if connected so we can use it for the next connetc
+     * we need to save this even if connected so we can use it for the next connect
      */
     if ( attribute == SQL_ATTR_LOGIN_TIMEOUT )
     {
@@ -536,7 +703,7 @@ SQLRETURN SQLSetConnectAttrW( SQLHDBC connection_handle,
                       case SQL_ATTR_TRANSLATE_LIB:
                         if ( value )
                         {
-                            as1 = (SQLCHAR*) unicode_to_ansi_alloc( value, SQL_NTS, connection );
+                            as1 = (SQLCHAR*) unicode_to_ansi_alloc( value, SQL_NTS, connection, NULL );
                         }
                         break;
                     }
@@ -576,24 +743,34 @@ SQLRETURN SQLSetConnectAttrW( SQLHDBC connection_handle,
                     {
                         if ( string_length > 0 )
                         {
-                            as1 = (SQLCHAR*) unicode_to_ansi_alloc( value, string_length, connection );
+                            as1 = (SQLCHAR*) unicode_to_ansi_alloc( value, string_length, connection, NULL );
                         }
                         else if ( string_length == SQL_NTS )
                         {
-                            as1 = (SQLCHAR*) unicode_to_ansi_alloc( value, SQL_NTS, connection );
+                            as1 = (SQLCHAR*) unicode_to_ansi_alloc( value, SQL_NTS, connection, NULL );
                         }
                     }
+
+                    ret = SQLSETCONNECTATTR( connection,
+                            connection -> driver_dbc,
+                            attribute,
+                            as1 ? as1 : value,
+                            string_length / sizeof( SQLWCHAR ));
+
+                    if ( as1 ) 
+                    {
+                        free( as1 );
+                    }
+                    break;
+
+                  default:
+                    ret = SQLSETCONNECTATTR( connection,
+                            connection -> driver_dbc,
+                            attribute,
+                            value,
+                            string_length );
                     break;
                 }
-
-                ret = SQLSETCONNECTATTR( connection,
-                        connection -> driver_dbc,
-                        attribute,
-                        as1 ? as1 : value,
-                        string_length );
-
-                if ( as1 )
-                    free( as1 );
             }
         }
 
