@@ -4,7 +4,7 @@
  * (pharvey@codebydesign.com).
  *
  * Modified and extended by Nick Gorham
- * (nick@easysoft.com).
+ * (nick@lurcher.org).
  *
  * Any bugs or problems should be considered the fault of Nick and not
  * Peter.
@@ -27,9 +27,12 @@
  *
  **********************************************************************
  *
- * $Id: SQLSetCursorName.c,v 1.5 2003/10/30 18:20:46 lurcher Exp $
+ * $Id: SQLSetCursorName.c,v 1.6 2009/02/18 17:59:08 lurcher Exp $
  *
  * $Log: SQLSetCursorName.c,v $
+ * Revision 1.6  2009/02/18 17:59:08  lurcher
+ * Shift to using config.h, the compile lines were making it hard to spot warnings
+ *
  * Revision 1.5  2003/10/30 18:20:46  lurcher
  *
  * Fix broken thread protection
@@ -110,9 +113,10 @@
  *
  **********************************************************************/
 
+#include <config.h>
 #include "drivermanager.h"
 
-static char const rcsid[]= "$RCSfile: SQLSetCursorName.c,v $ $Revision: 1.5 $";
+static char const rcsid[]= "$RCSfile: SQLSetCursorName.c,v $ $Revision: 1.6 $";
 
 SQLRETURN SQLSetCursorNameA( SQLHSTMT statement_handle,
            SQLCHAR *cursor_name,
@@ -151,8 +155,8 @@ SQLRETURN SQLSetCursorName( SQLHSTMT statement_handle,
     if ( log_info.log_flag )
     {
         sprintf( statement -> msg, "\n\t\tEntry:\
-            \n\t\t\tStatement = %p\
-            \n\t\t\tCursor name = %s",
+\n\t\t\tStatement = %p\
+\n\t\t\tCursor name = %s",
                 statement,
                 __string_with_length( s1, cursor_name, name_length ));
 
@@ -206,7 +210,10 @@ SQLRETURN SQLSetCursorName( SQLHSTMT statement_handle,
             statement -> state == STATE_S9 ||
             statement -> state == STATE_S10 ||
             statement -> state == STATE_S11 ||
-            statement -> state == STATE_S12 )
+            statement -> state == STATE_S12 ||
+            statement -> state == STATE_S13 ||
+            statement -> state == STATE_S14 ||
+            statement -> state == STATE_S15 )
     {
         dm_log_write( __FILE__, 
                 __LINE__, 
@@ -224,6 +231,7 @@ SQLRETURN SQLSetCursorName( SQLHSTMT statement_handle,
     if ( statement -> connection -> unicode_driver )
     {
         SQLWCHAR *s1;
+        int wlen;
 
         if ( !CHECK_SQLSETCURSORNAMEW( statement -> connection ))
         {
@@ -240,7 +248,9 @@ SQLRETURN SQLSetCursorName( SQLHSTMT statement_handle,
             return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
         }
 
-        s1 = ansi_to_unicode_alloc( cursor_name, name_length, statement -> connection );
+        s1 = ansi_to_unicode_alloc( cursor_name, name_length, statement -> connection, &wlen );
+
+        name_length = wlen;
 
         ret = SQLSETCURSORNAMEW( statement -> connection,
                 statement -> driver_stmt,
