@@ -4,7 +4,7 @@
  * (pharvey@codebydesign.com).
  *
  * Modified and extended by Nick Gorham
- * (nick@easysoft.com).
+ * (nick@lurcher.org).
  *
  * Any bugs or problems should be considered the fault of Nick and not
  * Peter.
@@ -27,9 +27,15 @@
  *
  **********************************************************************
  *
- * $Id: SQLSetStmtAttrW.c,v 1.8 2008/08/29 08:01:39 lurcher Exp $
+ * $Id: SQLSetStmtAttrW.c,v 1.10 2009/02/18 17:59:08 lurcher Exp $
  *
  * $Log: SQLSetStmtAttrW.c,v $
+ * Revision 1.10  2009/02/18 17:59:08  lurcher
+ * Shift to using config.h, the compile lines were making it hard to spot warnings
+ *
+ * Revision 1.9  2009/02/04 09:30:02  lurcher
+ * Fix some SQLINTEGER/SQLLEN conflicts
+ *
  * Revision 1.8  2008/08/29 08:01:39  lurcher
  * Alter the way W functions are passed to the driver
  *
@@ -82,6 +88,7 @@
  *
  **********************************************************************/
 
+#include <config.h>
 #include "drivermanager.h"
 
 static char const rcsid[]= "$RCSfile: SQLSetStmtAttrW.c,v $";
@@ -146,10 +153,10 @@ SQLRETURN SQLSetStmtAttrW( SQLHSTMT statement_handle,
     if ( log_info.log_flag )
     {
         sprintf( statement -> msg, "\n\t\tEntry:\
-            \n\t\t\tStatement = %p\
-            \n\t\t\tAttribute = %s\
-            \n\t\t\tValue = %p\
-            \n\t\t\tStrLen = %d",
+\n\t\t\tStatement = %p\
+\n\t\t\tAttribute = %s\
+\n\t\t\tValue = %p\
+\n\t\t\tStrLen = %d",
                 statement,
                 __stmt_attr_as_string( s1, attribute ),
                 value, 
@@ -211,7 +218,10 @@ SQLRETURN SQLSetStmtAttrW( SQLHSTMT statement_handle,
                 statement -> state == STATE_S9 ||
                 statement -> state == STATE_S10 ||
                 statement -> state == STATE_S11 ||
-                statement -> state == STATE_S12 )
+                statement -> state == STATE_S12 ||
+                statement -> state == STATE_S13 ||
+                statement -> state == STATE_S14 ||
+                statement -> state == STATE_S15 )
         {
             if ( statement -> prepared )
             {
@@ -353,6 +363,7 @@ SQLRETURN SQLSetStmtAttrW( SQLHSTMT statement_handle,
          */
         value = ( SQLPOINTER ) desc -> driver_desc;
         statement -> ard = desc;
+        desc -> associated_with = statement;
     }
 
     if ( attribute == SQL_ATTR_APP_PARAM_DESC )
@@ -413,6 +424,7 @@ SQLRETURN SQLSetStmtAttrW( SQLHSTMT statement_handle,
          */
         value = ( SQLPOINTER ) desc -> driver_desc;
         statement -> apd = desc;
+        desc -> associated_with = statement;
     }
 
     /*
@@ -421,7 +433,43 @@ SQLRETURN SQLSetStmtAttrW( SQLHSTMT statement_handle,
 
     if ( attribute == SQL_ATTR_METADATA_ID )
     {
-        statement -> metadata_id = (long int) value;
+        statement -> metadata_id = (SQLINTEGER) value;
+    }
+
+    if ( attribute == SQL_ATTR_IMP_ROW_DESC || 
+        attribute == SQL_ATTR_IMP_PARAM_DESC )
+    {
+        dm_log_write( __FILE__, 
+                    __LINE__, 
+                    LOG_INFO, 
+                    LOG_INFO, 
+                    "Error: HY017" );
+
+        __post_internal_error( &statement -> error,
+                ERROR_HY017, NULL,
+                statement -> connection -> environment -> requested_version );
+
+        return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
+    }
+
+    /*
+     * is it a legitimate value
+     */
+    ret = dm_check_statement_attrs( statement, attribute, value );
+
+    if ( ret != SQL_SUCCESS ) 
+    {
+        dm_log_write( __FILE__, 
+                    __LINE__, 
+                    LOG_INFO, 
+                    LOG_INFO, 
+                    "Error: HY011" );
+
+        __post_internal_error( &statement -> error,
+                ERROR_HY024, NULL,
+                statement -> connection -> environment -> requested_version );
+
+        return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
     }
 
     /*
@@ -439,7 +487,7 @@ SQLRETURN SQLSetStmtAttrW( SQLHSTMT statement_handle,
             CHECK_SQLEXTENDEDFETCH( statement -> connection ) &&
             !CHECK_SQLFETCHSCROLL( statement -> connection ))
     {
-        statement -> fetch_bm_ptr = (SQLUINTEGER*) value;
+        statement -> fetch_bm_ptr = (SQLULEN*) value;
         ret = SQL_SUCCESS;
     }
     else if ( attribute == SQL_ATTR_ROW_STATUS_PTR &&
@@ -451,7 +499,7 @@ SQLRETURN SQLSetStmtAttrW( SQLHSTMT statement_handle,
     else if ( attribute == SQL_ATTR_ROWS_FETCHED_PTR &&
 	      statement -> connection -> driver_act_ver == SQL_OV_ODBC2 )
     {
-        statement -> row_ct_ptr = (SQLUINTEGER*) value;
+        statement -> row_ct_ptr = (SQLULEN*) value;
         ret = SQL_SUCCESS;
     }
     else if ( attribute == SQL_ATTR_ROW_ARRAY_SIZE &&
