@@ -4,7 +4,7 @@
  * (pharvey@codebydesign.com).
  *
  * Modified and extended by Nick Gorham
- * (nick@easysoft.com).
+ * (nick@lurcher.org).
  *
  * Any bugs or problems should be considered the fault of Nick and not
  * Peter.
@@ -27,9 +27,12 @@
  *
  **********************************************************************
  *
- * $Id: SQLStatisticsW.c,v 1.8 2008/08/29 08:01:39 lurcher Exp $
+ * $Id: SQLStatisticsW.c,v 1.9 2009/02/18 17:59:08 lurcher Exp $
  *
  * $Log: SQLStatisticsW.c,v $
+ * Revision 1.9  2009/02/18 17:59:08  lurcher
+ * Shift to using config.h, the compile lines were making it hard to spot warnings
+ *
  * Revision 1.8  2008/08/29 08:01:39  lurcher
  * Alter the way W functions are passed to the driver
  *
@@ -79,6 +82,7 @@
  *
  **********************************************************************/
 
+#include <config.h>
 #include "drivermanager.h"
 
 static char const rcsid[]= "$RCSfile: SQLStatisticsW.c,v $";
@@ -152,12 +156,12 @@ SQLRETURN SQLStatisticsW( SQLHSTMT statement_handle,
     if ( log_info.log_flag )
     {
         sprintf( statement -> msg, "\n\t\tEntry:\
-            \n\t\t\tStatement = %p\
-            \n\t\t\tCatalog Name = %s\
-            \n\t\t\tSchema Name = %s\
-            \n\t\t\tTable Name = %s\
-            \n\t\t\tUnique = %d\
-            \n\t\t\tReserved = %d",
+\n\t\t\tStatement = %p\
+\n\t\t\tCatalog Name = %s\
+\n\t\t\tSchema Name = %s\
+\n\t\t\tTable Name = %s\
+\n\t\t\tUnique = %d\
+\n\t\t\tReserved = %d",
                 statement,
                 __wstring_with_length( s1, catalog_name, name_length1 ), 
                 __wstring_with_length( s2, schema_name, name_length2 ), 
@@ -232,7 +236,7 @@ SQLRETURN SQLStatisticsW( SQLHSTMT statement_handle,
             statement -> state == STATE_S6 ||
             statement -> state == STATE_S7 )
 #else
-    if ( statement -> state == STATE_S6 ||
+    if (( statement -> state == STATE_S6 && statement -> eod == 0 ) ||
             statement -> state == STATE_S7 )
 #endif
     {
@@ -250,7 +254,10 @@ SQLRETURN SQLStatisticsW( SQLHSTMT statement_handle,
     }
     else if ( statement -> state == STATE_S8 ||
             statement -> state == STATE_S9 ||
-            statement -> state == STATE_S10 )
+            statement -> state == STATE_S10 ||
+            statement -> state == STATE_S13 ||
+            statement -> state == STATE_S14 ||
+            statement -> state == STATE_S15 )
     {
         dm_log_write( __FILE__, 
                 __LINE__, 
@@ -284,9 +291,37 @@ SQLRETURN SQLStatisticsW( SQLHSTMT statement_handle,
         }
     }
 
-    /*
-     * TO_DO Check the SQL_ATTR_METADATA_ID settings
-     */
+    if ( table_name == NULL ) 
+    {
+        dm_log_write( __FILE__, 
+                __LINE__, 
+                LOG_INFO, 
+                LOG_INFO, 
+                "Error: HY009" );
+
+        __post_internal_error( &statement -> error,
+                ERROR_HY009, NULL,
+                statement -> connection -> environment -> requested_version );
+
+        return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
+    }
+
+    if ( statement -> metadata_id == SQL_TRUE ) {
+        if ( schema_name == NULL ) 
+        {
+            dm_log_write( __FILE__, 
+                    __LINE__, 
+                    LOG_INFO, 
+                    LOG_INFO, 
+                    "Error: HY009" );
+    
+            __post_internal_error( &statement -> error,
+                    ERROR_HY009, NULL,
+                    statement -> connection -> environment -> requested_version );
+    
+            return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
+        }
+    }
 
     if ( statement -> connection -> unicode_driver ||
 		    CHECK_SQLSTATISTICSW( statement -> connection ))
@@ -320,6 +355,7 @@ SQLRETURN SQLStatisticsW( SQLHSTMT statement_handle,
     else
     {
         SQLCHAR *as1, *as2, *as3;
+        int clen;
 
         if ( !CHECK_SQLSTATISTICS( statement -> connection ))
         {
@@ -336,9 +372,12 @@ SQLRETURN SQLStatisticsW( SQLHSTMT statement_handle,
             return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
         }
 
-        as1 = (SQLCHAR*) unicode_to_ansi_alloc( catalog_name, name_length1, statement -> connection );
-        as2 = (SQLCHAR*) unicode_to_ansi_alloc( schema_name, name_length2, statement -> connection );
-        as3 = (SQLCHAR*) unicode_to_ansi_alloc( table_name, name_length3, statement -> connection );
+        as1 = (SQLCHAR*) unicode_to_ansi_alloc( catalog_name, name_length1, statement -> connection, &clen );
+        name_length1 = clen;
+        as2 = (SQLCHAR*) unicode_to_ansi_alloc( schema_name, name_length2, statement -> connection, &clen );
+        name_length2 = clen;
+        as3 = (SQLCHAR*) unicode_to_ansi_alloc( table_name, name_length3, statement -> connection, &clen );
+        name_length3 = clen;
 
         ret = SQLSTATISTICS( statement -> connection,
                 statement -> driver_stmt,
