@@ -28,8 +28,8 @@ static BOOL SQLConfigDataSourceWide(	HWND	hWnd,
 	void 	*hDLL	= FALSE;
 	HINI	hIni;
 	char	szDriverSetup[INI_MAX_PROPERTY_VALUE+1];
-    char    szIniName[ INI_MAX_OBJECT_NAME + 1 ];
-	char	b1[ 256 ], b2[ 256 ];
+    char    szIniName[ ODBC_FILENAME_MAX * 2 + 3 ];
+	char	b1[ ODBC_FILENAME_MAX + 1 ], b2[ ODBC_FILENAME_MAX + 1 ];
 
 	/* SANITY CHECKS */
 	if ( pszDriver == NULL )
@@ -37,7 +37,7 @@ static BOOL SQLConfigDataSourceWide(	HWND	hWnd,
 		inst_logPushMsg( __FILE__, __FILE__, __LINE__, LOG_CRITICAL, ODBC_ERROR_GENERAL_ERR, "" );
 		return FALSE;
 	}
-	
+
 	if ( pszDriver[0] == '\0' )
 	{
 		inst_logPushMsg( __FILE__, __FILE__, __LINE__, LOG_CRITICAL, ODBC_ERROR_GENERAL_ERR, "" );
